import p from "node:path";
import postcss from "postcss";
import inlineSVG from "postcss-inline-svg";
import chalk from "chalk";
import { readFileSync, existsSync } from "node:fs";
import { mkdir, rm, readFile, writeFile } from "node:fs/promises";
import * as esbuild from "esbuild";
import path from "path";
import tailwindcss from "@tailwindcss/postcss";
import { cwd } from "node:process";
import { exit } from "node:process";
import { exec } from "child_process";

export const tailwind = async (env) => {
  console.log(chalk.hex("#31aff6").bold(`⚙️ Recompiling tailwind.css`));

  await Promise.all(
    [
      `npx @tailwindcss/cli -i global/tailwind.css -o dist/tailwind.css`,
      `npx @tailwindcss/cli -i global/ck-tailwind.css -o dist/ck-tailwind.css`,
    ].map((command) => {
      return new Promise((resolve, reject) => {
        exec(command, async (err, stdout, stderr) => {
          if (err) {
            console.error(`Error processing ${command}:`, stderr);
            reject(err);
          } else {
            resolve(stdout);
          }
        });
      });
    }),
  ).then(() => {
    console.log(chalk.hex("#31aff6").bold(`✅ Compiled tailwind.css`));
  });
};

export const css = async (source, destination, env) => {
  const filePath = [source.dir, source.base].join("/");
  const outputFile = `${destination}/${source.name}.css`;

  console.log(chalk.hex("#987ae5").bold(`⚙️ Compiling: ${filePath}`));

  const css = await readFileSync(filePath, "utf8");

  const post = await postcss([
    tailwindcss({
      base: cwd(),
    }),
    inlineSVG({
      paths: [
        "global/",
        "components",
        "node_modules/@material-symbols/svg-400/rounded",
      ],
    }),
  ]).process(css, {
    from: filePath,
  });

  await write(outputFile, post.css);

  console.log(chalk.hex("#6b1feb").bold(`✅ Compiled: ${outputFile}`));
  return;
};

export const js = async (source, destination, env) => {
  const filePath = [source.dir, source.base].join("/");
  const outputFile = `${destination}/${source.name}.js`;
  console.log(chalk.hex("#ffdc49").bold(`⚙️ Compiling: ${filePath}`));
  await esbuild.build({
    entryPoints: [filePath],
    bundle: false,
    minify: env === "development" ? false : true,
    sourcemap: env === "development" ? true : false,
    target: ["chrome58", "firefox57"],
    outfile: outputFile,
  });
  console.log(chalk.hex("#fcbc0a").bold(`✅ Compiled: ${outputFile}`));
};

export const write = async (file, data) => {
  //console.log(`Writing ${file}`)
  const dir = path.dirname(file);
  await mkdir(dir, { recursive: true });
  return await writeFile(file, data);
};

export const process = async (path, env = "production") => {
  const props = p.parse(path);
  const dest = props.dir.replace("src", "dist");

  switch (props.ext) {
    case ".twig":
      if (env === "development") {
        await tailwind(env);
      }
      break;
    case ".pcss":
      await css(props, dest, env);
      break;
    case ".tsx":
    case ".ts":
      await js(props, dest, env);
      break;
    default:
      //console.log(props.ext)
      break;
  }
};
